@@ -7,7 +7,6 @@ from collections import namedtuple
 from itertools import product, combinations
 import asyncio
 import pytz
-import dateparser
 
 from vectorbt import settings
 from vectorbt.utils import checks, config, decorators, math, array, random, enum, params, attr, datetime, schedule
@@ -28,10 +27,10 @@ class TestConfig:
         conf = config.Config({'a': 0, 'b': {'c': 1}}, frozen=True)
         conf['a'] = 2
 
-        with pytest.raises(Exception) as e_info:
+        with pytest.raises(Exception):
             conf['d'] = 2
 
-        with pytest.raises(Exception) as e_info:
+        with pytest.raises(Exception):
             conf.update(d=2)
 
         conf.update(d=2, force_update=True)
@@ -39,22 +38,22 @@ class TestConfig:
 
         conf = config.Config({'a': 0, 'b': {'c': 1}}, read_only=True)
 
-        with pytest.raises(Exception) as e_info:
+        with pytest.raises(Exception):
             conf['a'] = 2
 
-        with pytest.raises(Exception) as e_info:
+        with pytest.raises(Exception):
             del conf['a']
 
-        with pytest.raises(Exception) as e_info:
+        with pytest.raises(Exception):
             conf.pop('a')
 
-        with pytest.raises(Exception) as e_info:
+        with pytest.raises(Exception):
             conf.popitem()
 
-        with pytest.raises(Exception) as e_info:
+        with pytest.raises(Exception):
             conf.clear()
 
-        with pytest.raises(Exception) as e_info:
+        with pytest.raises(Exception):
             conf.update(a=2)
 
         assert isinstance(conf.merge_with(dict(b=dict(d=2))), config.Config)
@@ -488,7 +487,7 @@ class TestDecorators:
 
         class G:
             @decorators.cached_method
-            def cache_me(self, b=10): return np.random.uniform()
+            def cache_me(self, b=None): return np.random.uniform()
 
         g = G()
         cached_number = g.cache_me
@@ -496,7 +495,7 @@ class TestDecorators:
 
         class G:
             @decorators.cached_method(hello="world", hello2="world2")
-            def cache_me(self, b=10): return np.random.uniform()
+            def cache_me(self, b=None): return np.random.uniform()
 
         assert 'hello' in G.cache_me.kwargs
         assert G.cache_me.kwargs['hello'] == 'world'
@@ -895,13 +894,13 @@ class TestAttrs:
             def c(self):
                 return 0
 
-        with pytest.raises(Exception) as e_info:
+        with pytest.raises(Exception):
             _ = attr.deep_getattr(A(), 'a')
-        with pytest.raises(Exception) as e_info:
+        with pytest.raises(Exception):
             _ = attr.deep_getattr(A(), ('a',))
-        with pytest.raises(Exception) as e_info:
+        with pytest.raises(Exception):
             _ = attr.deep_getattr(A(), ('a', 1))
-        with pytest.raises(Exception) as e_info:
+        with pytest.raises(Exception):
             _ = attr.deep_getattr(A(), ('a', (1,)))
         assert attr.deep_getattr(A(), ('a', (1,), {'y': 1})) == 2
         assert attr.deep_getattr(C(), 'c') == 0
@@ -1139,7 +1138,7 @@ class TestChecks:
 
     def test_assert_in(self):
         checks.assert_in(0, (0, 1))
-        with pytest.raises(Exception) as e_info:
+        with pytest.raises(Exception):
             checks.assert_in(2, (0, 1))
 
     def test_assert_numba_func(self):
@@ -1151,19 +1150,19 @@ class TestChecks:
             return x
 
         checks.assert_numba_func(test_func_nb)
-        with pytest.raises(Exception) as e_info:
+        with pytest.raises(Exception):
             checks.assert_numba_func(test_func)
 
     def test_assert_not_none(self):
         checks.assert_not_none(0)
-        with pytest.raises(Exception) as e_info:
+        with pytest.raises(Exception):
             checks.assert_not_none(None)
 
     def test_assert_type(self):
         checks.assert_type(0, int)
         checks.assert_type(np.zeros(1), (np.ndarray, pd.Series))
         checks.assert_type(pd.Series([1, 2, 3]), (np.ndarray, pd.Series))
-        with pytest.raises(Exception) as e_info:
+        with pytest.raises(Exception):
             checks.assert_type(pd.DataFrame([1, 2, 3]), (np.ndarray, pd.Series))
 
     def test_assert_subclass(self):
@@ -1179,36 +1178,36 @@ class TestChecks:
         checks.assert_subclass(B, A)
         checks.assert_subclass(C, B)
         checks.assert_subclass(C, A)
-        with pytest.raises(Exception) as e_info:
+        with pytest.raises(Exception):
             checks.assert_subclass(A, B)
 
     def test_assert_type_equal(self):
         checks.assert_type_equal(0, 1)
         checks.assert_type_equal(np.zeros(1), np.empty(1))
-        with pytest.raises(Exception) as e_info:
+        with pytest.raises(Exception):
             checks.assert_type(0, np.zeros(1))
 
     def test_assert_dtype(self):
-        checks.assert_dtype(np.zeros(1), np.float)
-        checks.assert_dtype(pd.Series([1, 2, 3]), np.int)
-        checks.assert_dtype(pd.DataFrame({'a': [1, 2], 'b': [3, 4]}), np.int)
-        with pytest.raises(Exception) as e_info:
-            checks.assert_dtype(pd.DataFrame({'a': [1, 2], 'b': [3., 4.]}), np.int)
+        checks.assert_dtype(np.zeros(1), np.float_)
+        checks.assert_dtype(pd.Series([1, 2, 3]), np.int_)
+        checks.assert_dtype(pd.DataFrame({'a': [1, 2], 'b': [3, 4]}), np.int_)
+        with pytest.raises(Exception):
+            checks.assert_dtype(pd.DataFrame({'a': [1, 2], 'b': [3., 4.]}), np.int_)
 
     def test_assert_subdtype(self):
         checks.assert_subdtype([0], np.number)
         checks.assert_subdtype(np.array([1, 2, 3]), np.number)
         checks.assert_subdtype(pd.DataFrame({'a': [1, 2], 'b': [3., 4.]}), np.number)
-        with pytest.raises(Exception) as e_info:
-            checks.assert_subdtype(np.array([1, 2, 3]), np.float)
-        with pytest.raises(Exception) as e_info:
-            checks.assert_subdtype(pd.DataFrame({'a': [1, 2], 'b': [3., 4.]}), np.float)
+        with pytest.raises(Exception):
+            checks.assert_subdtype(np.array([1, 2, 3]), np.floating)
+        with pytest.raises(Exception):
+            checks.assert_subdtype(pd.DataFrame({'a': [1, 2], 'b': [3., 4.]}), np.floating)
 
     def test_assert_dtype_equal(self):
         checks.assert_dtype_equal([1], [1, 1, 1])
         checks.assert_dtype_equal(pd.Series([1, 2, 3]), pd.DataFrame([[1, 2, 3]]))
         checks.assert_dtype_equal(pd.DataFrame([[1, 2, 3.]]), pd.DataFrame([[1, 2, 3.]]))
-        with pytest.raises(Exception) as e_info:
+        with pytest.raises(Exception):
             checks.assert_dtype_equal(pd.DataFrame([[1, 2, 3]]), pd.DataFrame([[1, 2, 3.]]))
 
     def test_assert_ndim(self):
@@ -1216,13 +1215,13 @@ class TestChecks:
         checks.assert_ndim(np.zeros(1), 1)
         checks.assert_ndim(pd.Series([1, 2, 3]), (1, 2))
         checks.assert_ndim(pd.DataFrame([1, 2, 3]), (1, 2))
-        with pytest.raises(Exception) as e_info:
+        with pytest.raises(Exception):
             checks.assert_ndim(np.zeros((3, 3, 3)), (1, 2))
 
     def test_assert_len_equal(self):
         checks.assert_len_equal([[1]], [[2]])
         checks.assert_len_equal([[1]], [[2, 3]])
-        with pytest.raises(Exception) as e_info:
+        with pytest.raises(Exception):
             checks.assert_len_equal([[1]], [[2], [3]])
 
     def test_assert_shape_equal(self):
@@ -1231,12 +1230,12 @@ class TestChecks:
         checks.assert_shape_equal([1, 2, 3], pd.Series([1, 2, 3]))
         checks.assert_shape_equal(np.zeros((3, 3)), pd.Series([1, 2, 3]), axis=0)
         checks.assert_shape_equal(np.zeros((2, 3)), pd.Series([1, 2, 3]), axis=(1, 0))
-        with pytest.raises(Exception) as e_info:
+        with pytest.raises(Exception):
             checks.assert_shape_equal(np.zeros((2, 3)), pd.Series([1, 2, 3]), axis=(0, 1))
 
     def test_assert_index_equal(self):
         checks.assert_index_equal(pd.Index([1, 2, 3]), pd.Index([1, 2, 3]))
-        with pytest.raises(Exception) as e_info:
+        with pytest.raises(Exception):
             checks.assert_index_equal(pd.Index([1, 2, 3]), pd.Index([2, 3, 4]))
 
     def test_assert_meta_equal(self):
@@ -1245,16 +1244,16 @@ class TestChecks:
         checks.assert_meta_equal(np.array([1, 2, 3]), np.array([1, 2, 3]))
         checks.assert_meta_equal(pd.Series([1, 2, 3], index=index), pd.Series([1, 2, 3], index=index))
         checks.assert_meta_equal(pd.DataFrame([[1, 2, 3]], columns=columns), pd.DataFrame([[1, 2, 3]], columns=columns))
-        with pytest.raises(Exception) as e_info:
+        with pytest.raises(Exception):
             checks.assert_meta_equal(pd.Series([1, 2]), pd.DataFrame([1, 2]))
 
-        with pytest.raises(Exception) as e_info:
+        with pytest.raises(Exception):
             checks.assert_meta_equal(pd.DataFrame([1, 2]), pd.DataFrame([1, 2, 3]))
 
-        with pytest.raises(Exception) as e_info:
+        with pytest.raises(Exception):
             checks.assert_meta_equal(pd.DataFrame([1, 2, 3]), pd.DataFrame([1, 2, 3], index=index))
 
-        with pytest.raises(Exception) as e_info:
+        with pytest.raises(Exception):
             checks.assert_meta_equal(pd.DataFrame([[1, 2, 3]]), pd.DataFrame([[1, 2, 3]], columns=columns))
 
     def test_assert_array_equal(self):
@@ -1264,7 +1263,7 @@ class TestChecks:
         checks.assert_array_equal(pd.Series([1, 2, 3], index=index), pd.Series([1, 2, 3], index=index))
         checks.assert_array_equal(pd.DataFrame([[1, 2, 3]], columns=columns),
                                   pd.DataFrame([[1, 2, 3]], columns=columns))
-        with pytest.raises(Exception) as e_info:
+        with pytest.raises(Exception):
             checks.assert_array_equal(np.array([1, 2]), np.array([1, 2, 3]))
 
     def test_assert_level_not_exists(self):
@@ -1272,22 +1271,22 @@ class TestChecks:
         multi_i = pd.MultiIndex.from_arrays([['x', 'y', 'z'], ['x2', 'y2', 'z2']], names=['i', 'i2'])
         checks.assert_level_not_exists(i, 'i2')
         checks.assert_level_not_exists(multi_i, 'i3')
-        with pytest.raises(Exception) as e_info:
+        with pytest.raises(Exception):
             checks.assert_level_not_exists(i, 'i')
             checks.assert_level_not_exists(multi_i, 'i')
 
     def test_assert_equal(self):
         checks.assert_equal(0, 0)
         checks.assert_equal(False, False)
-        with pytest.raises(Exception) as e_info:
+        with pytest.raises(Exception):
             checks.assert_equal(0, 1)
 
     def test_assert_dict_valid(self):
         checks.assert_dict_valid(dict(a=2, b=3), [['a', 'b', 'c']])
-        with pytest.raises(Exception) as e_info:
+        with pytest.raises(Exception):
             checks.assert_dict_valid(dict(a=2, b=3, d=4), [['a', 'b', 'c']])
         checks.assert_dict_valid(dict(a=2, b=3, c=dict(d=4, e=5)), [['a', 'b', 'c'], ['d', 'e']])
-        with pytest.raises(Exception) as e_info:
+        with pytest.raises(Exception):
             checks.assert_dict_valid(dict(a=2, b=3, c=dict(d=4, f=5)), [['a', 'b', 'c'], ['d', 'e']])
 
 
@@ -1443,42 +1442,42 @@ class TestArray:
         assert np.sum(array.uniform_summing_to_one_nb(10)) == 1
 
     def test_renormalize(self):
-        assert array.renormalize(0, [0, 10], [0, 1]) == 0
-        assert array.renormalize(10, [0, 10], [0, 1]) == 1
+        assert array.renormalize(0, (0, 10), (0, 1)) == 0
+        assert array.renormalize(10, (0, 10), (0, 1)) == 1
         np.testing.assert_array_equal(
-            array.renormalize(np.array([0, 2, 4, 6, 8, 10]), [0, 10], [0, 1]),
+            array.renormalize(np.array([0, 2, 4, 6, 8, 10]), (0, 10), (0, 1)),
             np.array([0., 0.2, 0.4, 0.6, 0.8, 1.])
         )
         np.testing.assert_array_equal(
-            array.renormalize_nb(np.array([0, 2, 4, 6, 8, 10]), [0, 10], [0, 1]),
+            array.renormalize_nb(np.array([0, 2, 4, 6, 8, 10]), (0, 10), (0, 1)),
             np.array([0., 0.2, 0.4, 0.6, 0.8, 1.])
         )
 
     def test_min_rel_rescale(self):
         np.testing.assert_array_equal(
-            array.min_rel_rescale(np.array([2, 4, 6]), [10, 20]),
+            array.min_rel_rescale(np.array([2, 4, 6]), (10, 20)),
             np.array([10., 15., 20.])
         )
         np.testing.assert_array_equal(
-            array.min_rel_rescale(np.array([5, 6, 7]), [10, 20]),
+            array.min_rel_rescale(np.array([5, 6, 7]), (10, 20)),
             np.array([10., 12., 14.])
         )
         np.testing.assert_array_equal(
-            array.min_rel_rescale(np.array([5, 5, 5]), [10, 20]),
+            array.min_rel_rescale(np.array([5, 5, 5]), (10, 20)),
             np.array([10., 10., 10.])
         )
 
     def test_max_rel_rescale(self):
         np.testing.assert_array_equal(
-            array.max_rel_rescale(np.array([2, 4, 6]), [10, 20]),
+            array.max_rel_rescale(np.array([2, 4, 6]), (10, 20)),
             np.array([10., 15., 20.])
         )
         np.testing.assert_array_equal(
-            array.max_rel_rescale(np.array([5, 6, 7]), [10, 20]),
+            array.max_rel_rescale(np.array([5, 6, 7]), (10, 20)),
             np.array([14.285714285714286, 17.142857142857142, 20.])
         )
         np.testing.assert_array_equal(
-            array.max_rel_rescale(np.array([5, 5, 5]), [10, 20]),
+            array.max_rel_rescale(np.array([5, 5, 5]), (10, 20)),
             np.array([20., 20., 20.])
         )
 
@@ -1489,10 +1488,10 @@ class TestArray:
 
         set_seed()
         np.testing.assert_array_equal(
-            array.rescale_float_to_int_nb(np.array([0.3, 0.3, 0.3, 0.1]), [10, 20], 70),
+            array.rescale_float_to_int_nb(np.array([0.3, 0.3, 0.3, 0.1]), (10, 20), 70),
             np.array([17, 14, 22, 17])
         )
-        assert np.sum(array.rescale_float_to_int_nb(np.array([0.3, 0.3, 0.3, 0.1]), [10, 20], 70)) == 70
+        assert np.sum(array.rescale_float_to_int_nb(np.array([0.3, 0.3, 0.3, 0.1]), (10, 20), 70)) == 70
 
 
 # ############# random.py ############# #
@@ -1521,22 +1520,22 @@ Enum = namedtuple('Enum', ['Attr1', 'Attr2'])(*range(2))
 
 
 class TestEnum:
-    def test_caseins_getattr(self):
-        assert enum.caseins_getattr(Enum, 'Attr1') == 0
-        assert enum.caseins_getattr(Enum, 'attr1') == 0
-        assert enum.caseins_getattr(Enum, 'Attr2') == 1
-        assert enum.caseins_getattr(Enum, 'attr2') == 1
-        with pytest.raises(Exception) as e_info:
-            enum.caseins_getattr(Enum, 'Attr3')
+    def test_get_caseins_enum_attr(self):
+        assert enum.get_caseins_enum_attr(Enum, 'Attr1') == 0
+        assert enum.get_caseins_enum_attr(Enum, 'attr1') == 0
+        assert enum.get_caseins_enum_attr(Enum, 'Attr2') == 1
+        assert enum.get_caseins_enum_attr(Enum, 'attr2') == 1
+        with pytest.raises(Exception):
+            enum.get_caseins_enum_attr(Enum, 'Attr3')
 
-    def test_convert_str_enum_value(self):
-        assert enum.convert_str_enum_value(Enum, 0) == 0
-        assert enum.convert_str_enum_value(Enum, 10) == 10
-        assert enum.convert_str_enum_value(Enum, 10.) == 10.
-        assert enum.convert_str_enum_value(Enum, 'Attr1') == 0
-        assert enum.convert_str_enum_value(Enum, 'attr1') == 0
-        assert enum.convert_str_enum_value(Enum, ('attr1', 'attr2')) == (0, 1)
-        assert enum.convert_str_enum_value(Enum, [['attr1', 'attr2']]) == [[0, 1]]
+    def test_prepare_enum_value(self):
+        assert enum.prepare_enum_value(Enum, 0) == 0
+        assert enum.prepare_enum_value(Enum, 10) == 10
+        assert enum.prepare_enum_value(Enum, 10.) == 10.
+        assert enum.prepare_enum_value(Enum, 'Attr1') == 0
+        assert enum.prepare_enum_value(Enum, 'attr1') == 0
+        assert enum.prepare_enum_value(Enum, ('attr1', 'attr2')) == (0, 1)
+        assert enum.prepare_enum_value(Enum, [['attr1', 'attr2']]) == [[0, 1]]
 
     def test_to_value_map(self):
         assert enum.to_value_map(Enum) == {-1: None, 0: 'Attr1', 1: 'Attr2'}
@@ -1570,7 +1569,7 @@ class TestParams:
 
 class TestDatetime:
     def test_to_timedelta(self):
-        assert datetime.to_timedelta('d') == pd.to_timedelta('1d')
+        assert datetime.freq_to_timedelta('d') == pd.to_timedelta('1d')
 
     def test_get_utc_tz(self):
         assert datetime.get_utc_tz().utcoffset(_datetime.now()) == _timedelta(0)
@@ -1604,14 +1603,15 @@ class TestDatetime:
         assert datetime.is_tz_aware(pd.Timestamp('2020-01-01', tz=datetime.get_utc_tz()))
 
     def test_to_timezone(self):
-        assert datetime.to_timezone('UTC') == pytz.UTC
-        assert datetime.to_timezone('Europe/Berlin') == pytz.timezone('Europe/Berlin')
+        assert datetime.to_timezone('UTC') == _timezone.utc
+        assert datetime.to_timezone('Europe/Berlin') == _timezone(_timedelta(hours=2))
+        assert datetime.to_timezone('Europe/Berlin', to_py_timezone=False) == pytz.timezone('Europe/Berlin')
         assert datetime.to_timezone('+0500') == _timezone(_timedelta(hours=5))
         assert datetime.to_timezone(_timezone(_timedelta(hours=1))) == _timezone(_timedelta(hours=1))
-        assert datetime.to_timezone(pytz.timezone('Europe/Berlin')) == pytz.timezone('Europe/Berlin')
+        assert datetime.to_timezone(pytz.timezone('Europe/Berlin')) == _timezone(_timedelta(hours=2))
         assert datetime.to_timezone(1) == _timezone(_timedelta(hours=1))
         assert datetime.to_timezone(0.5) == _timezone(_timedelta(hours=0.5))
-        with pytest.raises(Exception) as e_info:
+        with pytest.raises(Exception):
             _ = datetime.to_timezone('+05')
 
     def test_to_tzaware_datetime(self):
@@ -1634,7 +1634,7 @@ class TestDatetime:
         assert datetime.to_tzaware_datetime(
             _datetime(2020, 1, 1, 12, 0, 0, tzinfo=datetime.get_utc_tz()), tz=datetime.get_local_tz()) == \
                _datetime(2020, 1, 1, 12, 0, 0, tzinfo=datetime.get_utc_tz()).astimezone(datetime.get_local_tz())
-        with pytest.raises(Exception) as e_info:
+        with pytest.raises(Exception):
             _ = datetime.to_tzaware_datetime('2020-01-001')
 
     def test_datetime_to_ms(self):
