@@ -6,13 +6,22 @@ and cleansing MultiIndex levels. "Index" in pandas context is referred to both i
 import numpy as np
 import pandas as pd
 from numba import njit
-from collections.abc import Iterable
 from datetime import datetime, timedelta
 
+from vectorbt import typing as tp
 from vectorbt.utils import checks
 
 
-def get_index(arg, axis):
+def to_any_index(index_like: tp.IndexLike) -> tp.Index:
+    """Convert any index-like object to an index.
+
+    Index objects are kept as-is."""
+    if not isinstance(index_like, pd.Index):
+        return pd.Index(index_like)
+    return index_like
+
+
+def get_index(arg: tp.SeriesFrame, axis: int) -> tp.Index:
     """Get index of `arg` by `axis`."""
     checks.assert_type(arg, (pd.Series, pd.DataFrame))
     checks.assert_in(axis, (0, 1))
@@ -28,12 +37,10 @@ def get_index(arg, axis):
             return arg.columns
 
 
-def index_from_values(values, name=None):
+def index_from_values(values: tp.ArrayLikeSequence, name: tp.Optional[str] = None) -> tp.Index:
     """Create a new `pd.Index` with `name` by parsing an iterable `values`.
 
     Each in `values` will correspond to an element in the new index."""
-    checks.assert_type(values, Iterable)
-
     scalar_types = (int, float, complex, str, bool, datetime, timedelta, np.generic)
     value_names = []
     for i in range(len(values)):
@@ -41,7 +48,7 @@ def index_from_values(values, name=None):
         if v is None or isinstance(v, scalar_types):
             value_names.append(v)
         elif isinstance(v, np.ndarray):
-            if np.all(v == v.item(0)):
+            if (v == v.item(0)).all():
                 value_names.append(v.item(0))
             else:
                 value_names.append('array_%d' % i)
@@ -50,29 +57,31 @@ def index_from_values(values, name=None):
     return pd.Index(value_names, name=name)
 
 
-def repeat_index(index, n, ignore_default=None):
-    """Repeat each element in `index` `n` times."""
+def repeat_index(index: tp.IndexLike, n: int, ignore_default: tp.Optional[bool] = None) -> tp.Index:
+    """Repeat each element in `index` `n` times.
+
+    Set `ignore_default` to None to use the default."""
     from vectorbt import settings
 
     if ignore_default is None:
         ignore_default = settings.broadcasting['ignore_default']
 
-    if not isinstance(index, pd.Index):
-        index = pd.Index(index)
+    index = to_any_index(index)
     if checks.is_default_index(index) and ignore_default:  # ignore simple ranges without name
         return pd.RangeIndex(start=0, stop=len(index) * n, step=1)
-    return np.repeat(index, n)
+    return index.repeat(n)
 
 
-def tile_index(index, n, ignore_default=None):
-    """Tile the whole `index` `n` times."""
+def tile_index(index: tp.IndexLike, n: int, ignore_default: tp.Optional[bool] = None) -> tp.Index:
+    """Tile the whole `index` `n` times.
+
+    Set `ignore_default` to None to use the default."""
     from vectorbt import settings
 
     if ignore_default is None:
         ignore_default = settings.broadcasting['ignore_default']
 
-    if not isinstance(index, pd.Index):
-        index = pd.Index(index)
+    index = to_any_index(index)
     if checks.is_default_index(index) and ignore_default:  # ignore simple ranges without name
         return pd.RangeIndex(start=0, stop=len(index) * n, step=1)
     if isinstance(index, pd.MultiIndex):
@@ -80,12 +89,17 @@ def tile_index(index, n, ignore_default=None):
     return pd.Index(np.tile(index, n), name=index.name)
 
 
-def stack_indexes(indexes, drop_duplicates=None, keep=None, drop_redundant=None):
-    """Stack each index in `indexes` on top of each other, from top to bottom."""
+def stack_indexes(indexes: tp.Sequence[tp.IndexLike], drop_duplicates: tp.Optional[bool] = None,
+                  keep: tp.Optional[str] = None, drop_redundant: tp.Optional[bool] = None) -> tp.Index:
+    """Stack each index in `indexes` on top of each other, from top to bottom.
+
+    Set `drop_duplicates`, `keep`, or `drop_redundant` to None to use the default."""
     from vectorbt import settings
 
     if drop_duplicates is None:
         drop_duplicates = settings.broadcasting['drop_duplicates']
+    if keep is None:
+        keep = settings.broadcasting['keep']
     if drop_redundant is None:
         drop_redundant = settings.broadcasting['drop_redundant']
 
@@ -93,9 +107,7 @@ def stack_indexes(indexes, drop_duplicates=None, keep=None, drop_redundant=None)
     for i in range(len(indexes)):
         index = indexes[i]
         if not isinstance(index, pd.MultiIndex):
-            if not isinstance(index, pd.Index):
-                index = pd.Index(index)
-            levels.append(index)
+            levels.append(to_any_index(index))
         else:
             for j in range(index.nlevels):
                 levels.append(index.get_level_values(j))
@@ -108,27 +120,28 @@ def stack_indexes(indexes, drop_duplicates=None, keep=None, drop_redundant=None)
     return new_index
 
 
-def combine_indexes(indexes, ignore_default=None, **kwargs):
+def combine_indexes(indexes: tp.Sequence[tp.IndexLike],
+                    ignore_default: tp.Optional[bool] = None, **kwargs) -> tp.Index:
     """Combine each index in `indexes` using Cartesian product.
 
     Keyword arguments will be passed to `stack_indexes`."""
-    new_index = indexes[0]
+    new_index = to_any_index(indexes[0])
     for i in range(1, len(indexes)):
-        index1, index2 = new_index, indexes[i]
+        index1, index2 = new_index, to_any_index(indexes[i])
         new_index1 = repeat_index(index1, len(index2), ignore_default=ignore_default)
         new_index2 = tile_index(index2, len(index1), ignore_default=ignore_default)
         new_index = stack_indexes([new_index1, new_index2], **kwargs)
     return new_index
 
 
-def drop_levels(index, levels):
+def drop_levels(index: tp.Index, levels: tp.MaybeLevelSequence) -> tp.Index:
     """Softly drop `levels` in `index` by their name/position."""
     if not isinstance(index, pd.MultiIndex):
         return index
 
     levels_to_drop = []
-    if not isinstance(levels, (tuple, list)):
-        levels = [levels]
+    if isinstance(levels, (int, str)):
+        levels = (levels,)
     for level in levels:
         if level in index.names:
             if level not in levels_to_drop:
@@ -143,7 +156,7 @@ def drop_levels(index, levels):
     return index
 
 
-def rename_levels(index, name_dict):
+def rename_levels(index: tp.Index, name_dict: tp.Dict[str, tp.Any]) -> tp.Index:
     """Rename levels in `index` by `name_dict`."""
     for k, v in name_dict.items():
         if isinstance(index, pd.MultiIndex):
@@ -155,17 +168,17 @@ def rename_levels(index, name_dict):
     return index
 
 
-def select_levels(index, level_names):
+def select_levels(index: tp.Index, level_names: tp.MaybeLevelSequence) -> tp.Index:
     """Build a new index by selecting one or multiple `level_names` from `index`."""
     checks.assert_type(index, pd.MultiIndex)
 
-    if isinstance(level_names, (list, tuple)):
-        levels = [index.get_level_values(level_name) for level_name in level_names]
-        return pd.MultiIndex.from_arrays(levels)
-    return index.get_level_values(level_names)
+    if isinstance(level_names, (int, str)):
+        return index.get_level_values(level_names)
+    levels = [index.get_level_values(level_name) for level_name in level_names]
+    return pd.MultiIndex.from_arrays(levels)
 
 
-def drop_redundant_levels(index):
+def drop_redundant_levels(index: tp.Index) -> tp.Index:
     """Drop levels in `index` that either have a single unnamed value or a range from 0 to n."""
     if not isinstance(index, pd.MultiIndex):
         return index
@@ -182,23 +195,26 @@ def drop_redundant_levels(index):
     return index
 
 
-def drop_duplicate_levels(index, keep=None):
+def drop_duplicate_levels(index: tp.Index, keep: tp.Optional[str] = None) -> tp.Index:
     """Drop levels in `index` with the same name and values.
 
-    Set `keep` to 'last' to keep last levels, otherwise 'first'."""
+    Set `keep` to 'last' to keep last levels, otherwise 'first'.
+
+    Set `keep` to None to use the default."""
     from vectorbt import settings
 
     if keep is None:
         keep = settings.broadcasting['keep']
     if not isinstance(index, pd.MultiIndex):
         return index
+    checks.assert_in(keep.lower(), ['first', 'last'])
 
     levels = []
     levels_to_drop = []
     if keep == 'first':
         r = range(0, index.nlevels)
-    elif keep == 'last':
-        r = range(index.nlevels-1, -1, -1)  # loop backwards
+    else:
+        r = range(index.nlevels - 1, -1, -1)  # loop backwards
     for i in r:
         level = (index.levels[i].name, tuple(index.get_level_values(i).to_numpy().tolist()))
         if level not in levels:
@@ -208,8 +224,8 @@ def drop_duplicate_levels(index, keep=None):
     return index.droplevel(levels_to_drop)
 
 
-@njit
-def _align_index_to_nb(a, b):
+@njit(cache=True)
+def _align_index_to_nb(a: tp.Array1d, b: tp.Array1d) -> tp.Array1d:
     """Return indices required to align `a` to `b`."""
     idxs = np.empty(b.shape[0], dtype=np.int_)
     g = 0
@@ -222,7 +238,7 @@ def _align_index_to_nb(a, b):
     return idxs
 
 
-def align_index_to(index1, index2):
+def align_index_to(index1: tp.Index, index2: tp.Index) -> pd.IndexSlice:
     """Align `index1` to have the same shape as `index2` if they have any levels in common.
 
     Returns index slice for the aligning."""
@@ -277,7 +293,7 @@ def align_index_to(index1, index2):
     return pd.IndexSlice[_align_index_to_nb(unique1, unique2)]
 
 
-def align_indexes(indexes):
+def align_indexes(indexes: tp.Sequence[tp.Index]) -> tp.List[tp.Index]:
     """Align multiple indexes to each other."""
     max_len = max(map(len, indexes))
     indices = []
@@ -299,7 +315,12 @@ def align_indexes(indexes):
     return indices
 
 
-def pick_levels(index, required_levels=None, optional_levels=None):
+OptionalLevelSequence = tp.Optional[tp.Sequence[tp.Union[None, tp.Level]]]
+
+
+def pick_levels(index: tp.Index,
+                required_levels: OptionalLevelSequence = None,
+                optional_levels: OptionalLevelSequence = None) -> tp.Tuple[tp.List[int], tp.List[int]]:
     """Pick optional and required levels and return their indices.
 
     Raises an exception if index has less or more levels than expected."""
@@ -322,26 +343,32 @@ def pick_levels(index, required_levels=None, optional_levels=None):
     # Pick optional levels
     _optional_levels = []
     for level in optional_levels:
+        level_pos = None
         if level is not None:
             checks.assert_type(level, (int, str))
             if isinstance(level, str):
-                level = index.names.index(level)
-            if level < 0:
-                level = index.nlevels + level
-            levels_left.remove(level)
-        _optional_levels.append(level)
+                level_pos = index.names.index(level)
+            else:
+                level_pos = level
+            if level_pos < 0:
+                level_pos = index.nlevels + level_pos
+            levels_left.remove(level_pos)
+        _optional_levels.append(level_pos)
 
     # Pick required levels
     _required_levels = []
     for level in required_levels:
+        level_pos = None
         if level is not None:
             checks.assert_type(level, (int, str))
             if isinstance(level, str):
-                level = index.names.index(level)
-            if level < 0:
-                level = index.nlevels + level
-            levels_left.remove(level)
-        _required_levels.append(level)
+                level_pos = index.names.index(level)
+            else:
+                level_pos = level
+            if level_pos < 0:
+                level_pos = index.nlevels + level_pos
+            levels_left.remove(level_pos)
+        _required_levels.append(level_pos)
     for i, level in enumerate(_required_levels):
         if level is None:
             _required_levels[i] = levels_left.pop(0)
@@ -349,7 +376,7 @@ def pick_levels(index, required_levels=None, optional_levels=None):
     return _required_levels, _optional_levels
 
 
-def find_first_occurrence(index_value, index):
+def find_first_occurrence(index_value: tp.Any, index: tp.Index) -> int:
     """Return index of the first occurrence in `index`."""
     loc = index.get_loc(index_value)
     if isinstance(loc, slice):
