@@ -7,6 +7,7 @@ import time
 import warnings
 from functools import wraps
 
+from vectorbt import typing as tp
 from vectorbt.utils.datetime import (
     get_utc_tz,
     get_local_tz,
@@ -16,17 +17,32 @@ from vectorbt.utils.datetime import (
 from vectorbt.utils.config import merge_dicts, get_func_kwargs
 from vectorbt.data.base import Data
 
+try:
+    from binance.client import Client as ClientT
+except ImportError:
+    ClientT = tp.Any
+try:
+    from ccxt.base.exchange import Exchange as ExchangeT
+except ImportError:
+    ExchangeT = tp.Any
+
 
 class SyntheticData(Data):
     """`Data` for synthetically generated data."""
 
     @classmethod
-    def generate_symbol(cls, symbol, index, **kwargs):
+    def generate_symbol(cls, symbol: tp.Label, index: tp.Index, **kwargs) -> tp.SeriesFrame:
         """Abstract method to generate a symbol."""
         raise NotImplementedError
 
     @classmethod
-    def download_symbol(cls, symbol, start=0, end='now', freq=None, date_range_kwargs=None, **kwargs):
+    def download_symbol(cls,
+                        symbol: tp.Label,
+                        start: tp.DatetimeLike = 0,
+                        end: tp.DatetimeLike = 'now',
+                        freq: tp.Union[None, str, pd.DateOffset] = None,
+                        date_range_kwargs: tp.KwargsLike = None,
+                        **kwargs) -> tp.SeriesFrame:
         """Download the symbol.
 
         Generates datetime index and passes it to `SyntheticData.generate_symbol` to fill
@@ -34,8 +50,8 @@ class SyntheticData(Data):
         if date_range_kwargs is None:
             date_range_kwargs = {}
         index = pd.date_range(
-            start=to_tzaware_datetime(start),
-            end=to_tzaware_datetime(end),
+            start=to_tzaware_datetime(start, tz=get_utc_tz()),
+            end=to_tzaware_datetime(end, tz=get_utc_tz()),
             freq=freq,
             **date_range_kwargs
         )
@@ -43,7 +59,7 @@ class SyntheticData(Data):
             raise ValueError("Date range is empty")
         return cls.generate_symbol(symbol, index, **kwargs)
 
-    def update_symbol(self, symbol, **kwargs):
+    def update_symbol(self, symbol: tp.Label, **kwargs) -> tp.SeriesFrame:
         """Update the symbol.
 
         `**kwargs` will override keyword arguments passed to `SyntheticData.download_symbol`."""
@@ -53,7 +69,8 @@ class SyntheticData(Data):
         return self.download_symbol(symbol, **kwargs)
 
 
-def generate_gbm_paths(S0, mu, sigma, T, M, I, seed=None):
+def generate_gbm_paths(S0: float, mu: float, sigma: float, T: int, M: int, I: int,
+                       seed: tp.Optional[int] = None) -> tp.Array2d:
     """Generate using Geometric Brownian Motion (GBM).
 
     See https://stackoverflow.com/a/45036114/8141780."""
@@ -81,13 +98,13 @@ class GBMData(SyntheticData):
 
     >>> gbm_data = vbt.GBMData.download('GBM', start='2 hours ago', end='now', freq='1min', seed=42)
     >>> gbm_data.get()
-    2021-03-15 17:56:59.174370+01:00    102.386605
-    2021-03-15 17:57:59.174370+01:00    101.554203
-    2021-03-15 17:58:59.174370+01:00    104.765771
+    2021-05-02 14:14:15.182089+00:00    102.386605
+    2021-05-02 14:15:15.182089+00:00    101.554203
+    2021-05-02 14:16:15.182089+00:00    104.765771
     ...                                        ...
-    2021-03-15 19:54:59.174370+01:00     51.614839
-    2021-03-15 19:55:59.174370+01:00     53.525376
-    2021-03-15 19:56:59.174370+01:00     55.615250
+    2021-05-02 16:12:15.182089+00:00     51.614839
+    2021-05-02 16:13:15.182089+00:00     53.525376
+    2021-05-02 16:14:15.182089+00:00     55.615250
     Freq: T, Length: 121, dtype: float64
 
     >>> import time
@@ -95,18 +112,26 @@ class GBMData(SyntheticData):
 
     >>> gbm_data = gbm_data.update()
     >>> gbm_data.get()
-    2021-03-15 17:56:59.174370+01:00    102.386605
-    2021-03-15 17:57:59.174370+01:00    101.554203
-    2021-03-15 17:58:59.174370+01:00    104.765771
+    2021-05-02 14:14:15.182089+00:00    102.386605
+    2021-05-02 14:15:15.182089+00:00    101.554203
+    2021-05-02 14:16:15.182089+00:00    104.765771
     ...                                        ...
-    2021-03-15 19:55:59.174370+01:00     53.525376
-    2021-03-15 19:56:59.174370+01:00     51.082220
-    2021-03-15 19:57:59.174370+01:00     54.725304
+    2021-05-02 16:13:15.182089+00:00     53.525376
+    2021-05-02 16:14:15.182089+00:00     51.082220
+    2021-05-02 16:15:15.182089+00:00     54.725304
     Freq: T, Length: 122, dtype: float64
     ```"""
 
     @classmethod
-    def generate_symbol(cls, symbol, index, S0=100., mu=0., sigma=0.05, T=None, I=1, seed=None):
+    def generate_symbol(cls,
+                        symbol: tp.Label,
+                        index: tp.Index,
+                        S0: float = 100.,
+                        mu: float = 0.,
+                        sigma: float = 0.05,
+                        T: tp.Optional[int] = None,
+                        I: int = 1,
+                        seed: tp.Optional[int] = None) -> tp.SeriesFrame:
         """Generate the symbol using `generate_gbm_paths`.
 
         Args:
@@ -131,7 +156,7 @@ class GBMData(SyntheticData):
         columns = pd.RangeIndex(stop=out.shape[1], name='path')
         return pd.DataFrame(out, index=index, columns=columns)
 
-    def update_symbol(self, symbol, **kwargs):
+    def update_symbol(self, symbol: tp.Label, **kwargs) -> tp.SeriesFrame:
         """Update the symbol.
 
         `**kwargs` will override keyword arguments passed to `GBMData.download_symbol`."""
@@ -148,10 +173,12 @@ class GBMData(SyntheticData):
 class YFData(Data):
     """`Data` for data coming from `yfinance`.
 
-    !!! note
-        Sometimes `yfinance` returns a tz-naive datetime index. To produce a
-        tz-aware datetime index, pass `tz_localize` to `YFData.download`. In this case,
-        you would need to find a timezone that fits the data (+0500, +0000, etc.)
+    Stocks are usually in the timezone "+0500" and cryptocurrencies in UTC.
+
+    !!! warning
+        Data coming from Yahoo is not the most stable data out there. Yahoo may manipulate data
+        how they want, add noise, return missing data points (see volume in the example below), etc.
+        It's only used in vectorbt for demonstration purposes.
 
     ## Example
 
@@ -162,65 +189,69 @@ class YFData(Data):
 
     >>> yf_data = vbt.YFData.download(
     ...     "TSLA",
-    ...     start='2021-03-12 09:30:00 -0500',
-    ...     end='2021-03-12 15:55:00 -0500',
+    ...     start='2021-04-12 09:30:00 -0400',
+    ...     end='2021-04-12 09:35:00 -0400',
     ...     interval='1m'
     ... )
-    >>> yf_data.get()
-                                     Open        High         Low       Close  \\
+    >>> yf_data.get())
+                                     Open        High         Low       Close  \
     Datetime
-    2021-03-12 09:30:00-05:00  670.000000  674.455811  667.510010  673.445007
-    2021-03-12 09:31:00-05:00  673.424988  674.455811  668.099976  669.599976
-    2021-03-12 09:32:00-05:00  669.270020  673.000000  668.239990  670.929993
-    ...                               ...         ...         ...         ...
-    2021-03-12 15:52:00-05:00  690.630005  691.030029  690.380005  690.929993
-    2021-03-12 15:53:00-05:00  690.989990  692.200012  690.960022  692.039978
-    2021-03-12 15:54:00-05:00  692.000000  692.783875  691.929993  692.479980
+    2021-04-12 13:30:00+00:00  685.080017  685.679993  684.765015  685.679993
+    2021-04-12 13:31:00+00:00  684.625000  686.500000  684.010010  685.500000
+    2021-04-12 13:32:00+00:00  685.646790  686.820007  683.190002  686.455017
+    2021-04-12 13:33:00+00:00  686.455017  687.000000  685.000000  685.565002
+    2021-04-12 13:34:00+00:00  685.690002  686.400024  683.200012  683.715027
 
                                Volume  Dividends  Stock Splits
     Datetime
-    2021-03-12 09:30:00-05:00       0          0             0
-    2021-03-12 09:31:00-05:00  198906          0             0
-    2021-03-12 09:32:00-05:00  202002          0             0
-    ...                           ...        ...           ...
-    2021-03-12 15:52:00-05:00   97941          0             0
-    2021-03-12 15:53:00-05:00  155552          0             0
-    2021-03-12 15:54:00-05:00  110320          0             0
+    2021-04-12 13:30:00+00:00       0          0             0
+    2021-04-12 13:31:00+00:00  152276          0             0
+    2021-04-12 13:32:00+00:00  168363          0             0
+    2021-04-12 13:33:00+00:00  129607          0             0
+    2021-04-12 13:34:00+00:00  134620          0             0
 
-    [382 rows x 7 columns]
-
-    >>> yf_data = yf_data.update(end='2021-03-12 16:00:00 -0500')
+    >>> yf_data = yf_data.update(end='2021-04-12 09:40:00 -0400')
     >>> yf_data.get()
-                                     Open        High         Low       Close  \\
+                                     Open        High         Low       Close  \
     Datetime
-    2021-03-12 09:30:00-05:00  670.000000  674.455811  667.510010  673.445007
-    2021-03-12 09:31:00-05:00  673.424988  674.455811  668.099976  669.599976
-    2021-03-12 09:32:00-05:00  669.270020  673.000000  668.239990  670.929993
-    ...                               ...         ...         ...         ...
-    2021-03-12 15:57:00-05:00  693.239990  693.599976  693.039978  693.250000
-    2021-03-12 15:58:00-05:00  693.255005  693.419983  692.640015  692.950012
-    2021-03-12 15:59:00-05:00  692.909973  694.099976  692.570007  693.840027
+    2021-04-12 13:30:00+00:00  685.080017  685.679993  684.765015  685.679993
+    2021-04-12 13:31:00+00:00  684.625000  686.500000  684.010010  685.500000
+    2021-04-12 13:32:00+00:00  685.646790  686.820007  683.190002  686.455017
+    2021-04-12 13:33:00+00:00  686.455017  687.000000  685.000000  685.565002
+    2021-04-12 13:34:00+00:00  685.690002  686.400024  683.200012  683.715027
+    2021-04-12 13:35:00+00:00  683.604980  684.340027  682.760071  684.135010
+    2021-04-12 13:36:00+00:00  684.130005  686.640015  683.333984  686.563904
+    2021-04-12 13:37:00+00:00  686.530029  688.549988  686.000000  686.635010
+    2021-04-12 13:38:00+00:00  686.593201  689.500000  686.409973  688.179993
+    2021-04-12 13:39:00+00:00  688.500000  689.347595  687.710022  688.070007
 
                                Volume  Dividends  Stock Splits
     Datetime
-    2021-03-12 09:30:00-05:00       0          0             0
-    2021-03-12 09:31:00-05:00  198906          0             0
-    2021-03-12 09:32:00-05:00  202002          0             0
-    ...                           ...        ...           ...
-    2021-03-12 15:57:00-05:00  136808          0             0
-    2021-03-12 15:58:00-05:00  104432          0             0
-    2021-03-12 15:59:00-05:00  192474          0             0
-
-    [387 rows x 7 columns]
+    2021-04-12 13:30:00+00:00       0          0             0
+    2021-04-12 13:31:00+00:00  152276          0             0
+    2021-04-12 13:32:00+00:00  168363          0             0
+    2021-04-12 13:33:00+00:00  129607          0             0
+    2021-04-12 13:34:00+00:00       0          0             0
+    2021-04-12 13:35:00+00:00  110500          0             0
+    2021-04-12 13:36:00+00:00  148384          0             0
+    2021-04-12 13:37:00+00:00  243851          0             0
+    2021-04-12 13:38:00+00:00  203569          0             0
+    2021-04-12 13:39:00+00:00   93308          0             0
     ```
     """
 
     @classmethod
-    def download_symbol(cls, symbol, period='max', start=None, end=None, **kwargs):
+    def download_symbol(cls,
+                        symbol: tp.Label,
+                        period: str = 'max',
+                        start: tp.Optional[tp.DatetimeLike] = None,
+                        end: tp.Optional[tp.DatetimeLike] = None,
+                        **kwargs) -> tp.Frame:
         """Download the symbol.
 
         Args:
             symbol (str): Symbol.
+            period (str): Period.
             start (any): Start datetime.
 
                 See `vectorbt.utils.datetime.to_tzaware_datetime`.
@@ -239,7 +270,7 @@ class YFData(Data):
 
         return yf.Ticker(symbol).history(period=period, start=start, end=end, **kwargs)
 
-    def update_symbol(self, symbol, **kwargs):
+    def update_symbol(self, symbol: tp.Label, **kwargs) -> tp.Frame:
         """Update the symbol.
 
         `**kwargs` will override keyword arguments passed to `YFData.download_symbol`."""
@@ -249,6 +280,9 @@ class YFData(Data):
         return self.download_symbol(symbol, **kwargs)
 
 
+BinanceDataT = tp.TypeVar("BinanceDataT", bound="BinanceData")
+
+
 class BinanceData(Data):
     """`Data` for data coming from `python-binance`.
 
@@ -266,47 +300,46 @@ class BinanceData(Data):
     ...     interval='1m'
     ... )
     >>> binance_data.get()
-    2021-03-22 21:39:22.067000+00:00 - 2021-03-22 23:39:00+00:00: : 1it [00:00,  3.80it/s]
-
-                                   Open      High       Low     Close     Volume  \\
+    2021-05-02 14:47:20.478000+00:00 - 2021-05-02 16:47:00+00:00: : 1it [00:00,  3.42it/s]
+                                   Open      High       Low     Close     Volume  \
     Open time
-    2021-03-22 21:40:00+00:00  54676.30  54691.41  54532.77  54549.49  35.566780
-    2021-03-22 21:41:00+00:00  54547.36  54593.96  54543.23  54589.81  32.813574
-    2021-03-22 21:42:00+00:00  54589.81  54655.85  54585.57  54650.04  19.616003
+    2021-05-02 14:48:00+00:00  56867.44  56913.57  56857.40  56913.56  28.709976
+    2021-05-02 14:49:00+00:00  56913.56  56913.57  56845.94  56888.00  19.734841
+    2021-05-02 14:50:00+00:00  56888.00  56947.32  56879.78  56934.71  23.150163
     ...                             ...       ...       ...       ...        ...
-    2021-03-22 23:37:00+00:00  54443.83  54568.80  54403.49  54511.87  26.505448
-    2021-03-22 23:38:00+00:00  54511.86  54511.86  54388.00  54406.59  24.879893
-    2021-03-22 23:39:00+00:00  54411.55  54430.74  54386.70  54430.73   4.010454
+    2021-05-02 16:45:00+00:00  56664.13  56666.77  56641.11  56644.03  40.852719
+    2021-05-02 16:46:00+00:00  56644.02  56663.43  56605.17  56605.18  27.573654
+    2021-05-02 16:47:00+00:00  56605.18  56657.55  56605.17  56627.12   7.719933
 
-                                                    Close time  Quote volume  \\
+                                                    Close time  Quote volume  \
     Open time
-    2021-03-22 21:40:00+00:00 2021-03-22 21:40:59.999000+00:00  1.943891e+06
-    2021-03-22 21:41:00+00:00 2021-03-22 21:41:59.999000+00:00  1.790163e+06
-    2021-03-22 21:42:00+00:00 2021-03-22 21:42:59.999000+00:00  1.071824e+06
+    2021-05-02 14:48:00+00:00 2021-05-02 14:48:59.999000+00:00  1.633534e+06
+    2021-05-02 14:49:00+00:00 2021-05-02 14:49:59.999000+00:00  1.122519e+06
+    2021-05-02 14:50:00+00:00 2021-05-02 14:50:59.999000+00:00  1.317969e+06
     ...                                                    ...           ...
-    2021-03-22 23:37:00+00:00 2021-03-22 23:37:59.999000+00:00  1.444602e+06
-    2021-03-22 23:38:00+00:00 2021-03-22 23:38:59.999000+00:00  1.354316e+06
-    2021-03-22 23:39:00+00:00 2021-03-22 23:39:59.999000+00:00  2.182080e+05
+    2021-05-02 16:45:00+00:00 2021-05-02 16:45:59.999000+00:00  2.314579e+06
+    2021-05-02 16:46:00+00:00 2021-05-02 16:46:59.999000+00:00  1.561548e+06
+    2021-05-02 16:47:00+00:00 2021-05-02 16:47:59.999000+00:00  4.371848e+05
 
-                               Number of trades  Taker base volume  \\
+                               Number of trades  Taker base volume  \
     Open time
-    2021-03-22 21:40:00+00:00              1544          15.160921
-    2021-03-22 21:41:00+00:00              1638          15.132376
-    2021-03-22 21:42:00+00:00              1022          10.442487
+    2021-05-02 14:48:00+00:00               991          13.771152
+    2021-05-02 14:49:00+00:00               816           5.981942
+    2021-05-02 14:50:00+00:00              1086          10.813757
     ...                                     ...                ...
-    2021-03-22 23:37:00+00:00               795          13.412746
-    2021-03-22 23:38:00+00:00               815          10.808586
-    2021-03-22 23:39:00+00:00               209           2.312393
+    2021-05-02 16:45:00+00:00              1006          18.106933
+    2021-05-02 16:46:00+00:00               916          14.869411
+    2021-05-02 16:47:00+00:00               353           3.903321
 
                                Taker quote volume
     Open time
-    2021-03-22 21:40:00+00:00        8.288645e+05
-    2021-03-22 21:41:00+00:00        8.256025e+05
-    2021-03-22 21:42:00+00:00        5.706188e+05
+    2021-05-02 14:48:00+00:00        7.835391e+05
+    2021-05-02 14:49:00+00:00        3.402170e+05
+    2021-05-02 14:50:00+00:00        6.156418e+05
     ...                                       ...
-    2021-03-22 23:37:00+00:00        7.310994e+05
-    2021-03-22 23:38:00+00:00        5.883322e+05
-    2021-03-22 23:39:00+00:00        1.258189e+05
+    2021-05-02 16:45:00+00:00        1.025892e+06
+    2021-05-02 16:46:00+00:00        8.421173e+05
+    2021-05-02 16:47:00+00:00        2.210323e+05
 
     [120 rows x 10 columns]
 
@@ -315,51 +348,54 @@ class BinanceData(Data):
 
     >>> binance_data = binance_data.update()
     >>> binance_data.get()
-                                   Open      High       Low     Close     Volume  \\
+                                   Open      High       Low     Close     Volume  \
     Open time
-    2021-03-22 21:40:00+00:00  54676.30  54691.41  54532.77  54549.49  35.566780
-    2021-03-22 21:41:00+00:00  54547.36  54593.96  54543.23  54589.81  32.813574
-    2021-03-22 21:42:00+00:00  54589.81  54655.85  54585.57  54650.04  19.616003
+    2021-05-02 14:48:00+00:00  56867.44  56913.57  56857.40  56913.56  28.709976
+    2021-05-02 14:49:00+00:00  56913.56  56913.57  56845.94  56888.00  19.734841
+    2021-05-02 14:50:00+00:00  56888.00  56947.32  56879.78  56934.71  23.150163
     ...                             ...       ...       ...       ...        ...
-    2021-03-22 23:38:00+00:00  54511.86  54511.86  54388.00  54406.59  24.879893
-    2021-03-22 23:39:00+00:00  54411.55  54499.49  54386.70  54460.91  21.214675
-    2021-03-22 23:40:00+00:00  54460.90  54571.00  54458.12  54570.99  11.484854
+    2021-05-02 16:46:00+00:00  56644.02  56663.43  56605.17  56605.18  27.573654
+    2021-05-02 16:47:00+00:00  56605.18  56657.55  56605.17  56625.76  14.615437
+    2021-05-02 16:48:00+00:00  56625.75  56643.60  56614.32  56623.01   5.895843
 
-                                                    Close time  Quote volume  \\
+                                                    Close time  Quote volume  \
     Open time
-    2021-03-22 21:40:00+00:00 2021-03-22 21:40:59.999000+00:00  1.943891e+06
-    2021-03-22 21:41:00+00:00 2021-03-22 21:41:59.999000+00:00  1.790163e+06
-    2021-03-22 21:42:00+00:00 2021-03-22 21:42:59.999000+00:00  1.071824e+06
+    2021-05-02 14:48:00+00:00 2021-05-02 14:48:59.999000+00:00  1.633534e+06
+    2021-05-02 14:49:00+00:00 2021-05-02 14:49:59.999000+00:00  1.122519e+06
+    2021-05-02 14:50:00+00:00 2021-05-02 14:50:59.999000+00:00  1.317969e+06
     ...                                                    ...           ...
-    2021-03-22 23:38:00+00:00 2021-03-22 23:38:59.999000+00:00  1.354316e+06
-    2021-03-22 23:39:00+00:00 2021-03-22 23:39:59.999000+00:00  1.155268e+06
-    2021-03-22 23:40:00+00:00 2021-03-22 23:40:59.999000+00:00  6.262655e+05
+    2021-05-02 16:46:00+00:00 2021-05-02 16:46:59.999000+00:00  1.561548e+06
+    2021-05-02 16:47:00+00:00 2021-05-02 16:47:59.999000+00:00  8.276017e+05
+    2021-05-02 16:48:00+00:00 2021-05-02 16:48:59.999000+00:00  3.338702e+05
 
-                               Number of trades  Taker base volume  \\
+                               Number of trades  Taker base volume  \
     Open time
-    2021-03-22 21:40:00+00:00              1544          15.160921
-    2021-03-22 21:41:00+00:00              1638          15.132376
-    2021-03-22 21:42:00+00:00              1022          10.442487
+    2021-05-02 14:48:00+00:00               991          13.771152
+    2021-05-02 14:49:00+00:00               816           5.981942
+    2021-05-02 14:50:00+00:00              1086          10.813757
     ...                                     ...                ...
-    2021-03-22 23:38:00+00:00               815          10.808586
-    2021-03-22 23:39:00+00:00               693          10.565621
-    2021-03-22 23:40:00+00:00               448           6.107031
+    2021-05-02 16:46:00+00:00               916          14.869411
+    2021-05-02 16:47:00+00:00               912           7.778489
+    2021-05-02 16:48:00+00:00               308           2.358130
 
                                Taker quote volume
     Open time
-    2021-03-22 21:40:00+00:00        8.288645e+05
-    2021-03-22 21:41:00+00:00        8.256025e+05
-    2021-03-22 21:42:00+00:00        5.706188e+05
+    2021-05-02 14:48:00+00:00        7.835391e+05
+    2021-05-02 14:49:00+00:00        3.402170e+05
+    2021-05-02 14:50:00+00:00        6.156418e+05
     ...                                       ...
-    2021-03-22 23:38:00+00:00        5.883322e+05
-    2021-03-22 23:39:00+00:00        5.753598e+05
-    2021-03-22 23:40:00+00:00        3.330209e+05
+    2021-05-02 16:46:00+00:00        8.421173e+05
+    2021-05-02 16:47:00+00:00        4.404362e+05
+    2021-05-02 16:48:00+00:00        1.335474e+05
 
     [121 rows x 10 columns]
     ```"""
 
     @classmethod
-    def download(cls, symbols, client=None, **kwargs):
+    def download(cls: tp.Type[BinanceDataT],
+                 symbols: tp.Labels,
+                 client: tp.Optional["ClientT"] = None,
+                 **kwargs) -> BinanceDataT:
         """Override `vectorbt.data.base.Data.download` to instantiate a Binance client."""
         from binance.client import Client
         from vectorbt import settings
@@ -374,8 +410,15 @@ class BinanceData(Data):
         return super(BinanceData, cls).download(symbols, client=client, **kwargs)
 
     @classmethod
-    def download_symbol(cls, symbol, client=None, interval=None, start=0, end='now UTC',
-                        delay=500, limit=500, show_progress=True):
+    def download_symbol(cls,
+                        symbol: str,
+                        client: tp.Optional["ClientT"] = None,
+                        interval: tp.Optional[str] = None,
+                        start: tp.DatetimeLike = 0,
+                        end: tp.DatetimeLike = 'now UTC',
+                        delay: tp.Optional[float] = 500,
+                        limit: int = 500,
+                        show_progress: bool = True) -> tp.Frame:
         """Download the symbol.
 
         Args:
@@ -392,7 +435,7 @@ class BinanceData(Data):
             end (any): End datetime.
 
                 See `vectorbt.utils.datetime.to_tzaware_datetime`.
-            delay (int or float): Time to sleep after each request (in milliseconds).
+            delay (float): Time to sleep after each request (in milliseconds).
             limit (int): The maximum number of returned items.
             show_progress (bool): Whether to show the progress bar.
         """
@@ -415,11 +458,11 @@ class BinanceData(Data):
             next_start_ts = start_ts
         end_ts = datetime_to_ms(to_tzaware_datetime(end, tz=get_utc_tz()))
 
-        def _ts_to_str(ts):
+        def _ts_to_str(ts: tp.DatetimeLike) -> str:
             return str(pd.Timestamp(to_tzaware_datetime(ts, tz=get_utc_tz())))
 
         # Iteratively collect the data
-        data = []
+        data: tp.List[list] = []
         with tqdm(disable=not show_progress) as pbar:
             pbar.set_description(_ts_to_str(start_ts))
             while True:
@@ -480,7 +523,7 @@ class BinanceData(Data):
 
         return df
 
-    def update_symbol(self, symbol, **kwargs):
+    def update_symbol(self, symbol: str, **kwargs) -> tp.Frame:
         """Update the symbol.
 
         `**kwargs` will override keyword arguments passed to `BinanceData.download_symbol`."""
@@ -508,17 +551,16 @@ class CCXTData(Data):
     ...     timeframe='1m'
     ... )
     >>> ccxt_data.get()
-    2021-03-22 21:08:45.826000+00:00 - 2021-03-22 23:08:00+00:00: : 1it [00:00,  1.96it/s]
-
-                                   Open      High       Low     Close      Volume
+    2021-05-02 14:50:26.305000+00:00 - 2021-05-02 16:50:00+00:00: : 1it [00:00,  1.96it/s]
+                                   Open      High       Low     Close     Volume
     Open time
-    2021-03-22 21:09:00+00:00  54840.01  54843.01  54752.86  54819.23   34.311598
-    2021-03-22 21:10:00+00:00  54819.23  54868.27  54818.05  54857.81   33.024841
-    2021-03-22 21:11:00+00:00  54853.90  54917.13  54848.26  54915.80   30.612839
-    ...                             ...       ...       ...       ...         ...
-    2021-03-22 23:06:00+00:00  54802.16  54871.14  54802.16  54850.00   67.792298
-    2021-03-22 23:07:00+00:00  54850.00  54856.23  54771.89  54816.47   63.103731
-    2021-03-22 23:08:00+00:00  54816.45  54826.41  54733.86  54765.74   48.825476
+    2021-05-02 14:51:00+00:00  56934.70  56964.59  56910.00  56948.99  22.158319
+    2021-05-02 14:52:00+00:00  56948.99  56999.00  56940.04  56977.62  46.958464
+    2021-05-02 14:53:00+00:00  56977.61  56987.09  56882.98  56885.42  27.752200
+    ...                             ...       ...       ...       ...        ...
+    2021-05-02 16:48:00+00:00  56625.75  56643.60  56595.47  56596.01  15.452510
+    2021-05-02 16:49:00+00:00  56596.00  56664.14  56596.00  56640.35  12.777475
+    2021-05-02 16:50:00+00:00  56640.35  56675.82  56640.35  56670.65   6.882321
 
     [120 rows x 5 columns]
 
@@ -527,22 +569,32 @@ class CCXTData(Data):
 
     >>> ccxt_data = ccxt_data.update()
     >>> ccxt_data.get()
-                                   Open      High       Low     Close      Volume
+                                   Open      High       Low     Close     Volume
     Open time
-    2021-03-22 21:09:00+00:00  54840.01  54843.01  54752.86  54819.23   34.311598
-    2021-03-22 21:10:00+00:00  54819.23  54868.27  54818.05  54857.81   33.024841
-    2021-03-22 21:11:00+00:00  54853.90  54917.13  54848.26  54915.80   30.612839
-    ...                             ...       ...       ...       ...         ...
-    2021-03-22 23:07:00+00:00  54850.00  54856.23  54771.89  54816.47   63.103731
-    2021-03-22 23:08:00+00:00  54816.45  54826.41  54733.86  54777.12   74.730137
-    2021-03-22 23:09:00+00:00  54777.12  54869.48  54770.55  54827.52   45.687450
+    2021-05-02 14:51:00+00:00  56934.70  56964.59  56910.00  56948.99  22.158319
+    2021-05-02 14:52:00+00:00  56948.99  56999.00  56940.04  56977.62  46.958464
+    2021-05-02 14:53:00+00:00  56977.61  56987.09  56882.98  56885.42  27.752200
+    ...                             ...       ...       ...       ...        ...
+    2021-05-02 16:49:00+00:00  56596.00  56664.14  56596.00  56640.35  12.777475
+    2021-05-02 16:50:00+00:00  56640.35  56689.99  56640.35  56678.33  14.610231
+    2021-05-02 16:51:00+00:00  56678.33  56688.99  56636.89  56653.42  11.647158
 
     [121 rows x 5 columns]
     ```"""
 
     @classmethod
-    def download_symbol(cls, symbol, exchange='binance', config=None, timeframe='1d', start=0,
-                        end='now UTC', delay=None, limit=500, retries=3, show_progress=True, params=None):
+    def download_symbol(cls,
+                        symbol: str,
+                        exchange: tp.Union[str, "ExchangeT"] = 'binance',
+                        config: tp.Optional[dict] = None,
+                        timeframe: str = '1d',
+                        start: tp.DatetimeLike = 0,
+                        end: tp.DatetimeLike = 'now UTC',
+                        delay: tp.Optional[float] = None,
+                        limit: tp.Optional[int] = 500,
+                        retries: int = 3,
+                        show_progress: bool = True,
+                        params: tp.Optional[dict] = None) -> tp.Frame:
         """Download the symbol.
 
         Args:
@@ -561,7 +613,7 @@ class CCXTData(Data):
             end (any): End datetime.
 
                 See `vectorbt.utils.datetime.to_tzaware_datetime`.
-            delay (int or float): Time to sleep after each request (in milliseconds).
+            delay (float): Time to sleep after each request (in milliseconds).
 
                 !!! note
                     Use only if `enableRateLimit` is not set.
@@ -639,7 +691,7 @@ class CCXTData(Data):
             return str(pd.Timestamp(to_tzaware_datetime(ts, tz=get_utc_tz())))
 
         # Iteratively collect the data
-        data = []
+        data: tp.List[list] = []
         with tqdm(disable=not show_progress) as pbar:
             pbar.set_description(_ts_to_str(start_ts))
             while True:
@@ -682,7 +734,7 @@ class CCXTData(Data):
 
         return df
 
-    def update_symbol(self, symbol, **kwargs):
+    def update_symbol(self, symbol: str, **kwargs) -> tp.Frame:
         """Update the symbol.
 
         `**kwargs` will override keyword arguments passed to `CCXTData.download_symbol`."""
@@ -690,4 +742,4 @@ class CCXTData(Data):
         download_kwargs['start'] = self.data[symbol].index[-1]
         download_kwargs['show_progress'] = False
         kwargs = merge_dicts(download_kwargs, kwargs)
-        return self.download_symbol(symbol, **kwargs)
\ No newline at end of file
+        return self.download_symbol(symbol, **kwargs)
