@@ -2,22 +2,26 @@
 
 Defines enums and other schemas for `vectorbt.labels`."""
 
-from collections import namedtuple
 import json
 
+from vectorbt import typing as tp
+
 __all__ = [
     'TrendMode'
 ]
 
 __pdoc__ = {}
 
-TrendMode = namedtuple('TrendMode', [
-    'Binary',
-    'BinaryCont',
-    'BinaryContSat',
-    'PctChange',
-    'PctChangeNorm',
-])(*range(5))
+
+class TrendModeT(tp.NamedTuple):
+    Binary: int
+    BinaryCont: int
+    BinaryContSat: int
+    PctChange: int
+    PctChangeNorm: int
+
+
+TrendMode = TrendModeT(*range(5))
 """_"""
 
 __pdoc__['TrendMode'] = f"""Trend mode.
