@@ -16,21 +16,26 @@ import numpy as np
 import pandas as pd
 import plotly.graph_objects as go
 
+from vectorbt import typing as tp
 from vectorbt.utils.colors import adjust_lightness
 from vectorbt.utils.decorators import cached_property, cached_method
 from vectorbt.utils.config import merge_dicts
-from vectorbt.utils.datetime import DatetimeTypes
+from vectorbt.utils.datetime import DatetimeIndexes
 from vectorbt.utils.enum import to_value_map
-from vectorbt.utils.widgets import FigureWidget
+from vectorbt.utils.figure import make_figure
 from vectorbt.utils.array import min_rel_rescale, max_rel_rescale
 from vectorbt.base.reshape_fns import to_1d, to_2d, broadcast_to
+from vectorbt.base.array_wrapper import ArrayWrapper
 from vectorbt.records.base import Records
+from vectorbt.records.mapped_array import MappedArray
 from vectorbt.portfolio.enums import TradeDirection, TradeStatus, trade_dt, position_dt, TradeType
 from vectorbt.portfolio import nb
-
+from vectorbt.portfolio.orders import Orders
 
 # ############# Trades ############# #
 
+TradesT = tp.TypeVar("TradesT", bound="Trades")
+
 
 class Trades(Records):
     """Extends `Records` for working with trade records.
@@ -130,8 +135,13 @@ class Trades(Records):
     ```
     """
 
-    def __init__(self, wrapper, records_arr, close, idx_field='exit_idx',
-                 trade_type=TradeType.Trade, **kwargs):
+    def __init__(self,
+                 wrapper: ArrayWrapper,
+                 records_arr: tp.RecordArray,
+                 close: tp.ArrayLike,
+                 idx_field: str = 'exit_idx',
+                 trade_type: int = TradeType.Trade,
+                 **kwargs) -> None:
         Records.__init__(
             self,
             wrapper,
@@ -151,10 +161,11 @@ class Trades(Records):
             if not all(field in records_arr.dtype.names for field in position_dt.names):
                 raise TypeError("Records array must match position_dt")
 
-    def _indexing_func_meta(self, pd_indexing_func, **kwargs):
+    def indexing_func_meta(self: TradesT, pd_indexing_func: tp.PandasIndexingFunc,
+                           **kwargs) -> tp.Tuple[TradesT, tp.MaybeArray, tp.Array1d]:
         """Perform indexing on `Trades` and also return metadata."""
         new_wrapper, new_records_arr, group_idxs, col_idxs = \
-            Records._indexing_func_meta(self, pd_indexing_func, **kwargs)
+            Records.indexing_func_meta(self, pd_indexing_func, **kwargs)
         new_close = new_wrapper.wrap(to_2d(self.close, raw=True)[:, col_idxs], group_by=False)
         return self.copy(
             wrapper=new_wrapper,
@@ -162,22 +173,22 @@ class Trades(Records):
             close=new_close
         ), group_idxs, col_idxs
 
-    def _indexing_func(self, pd_indexing_func, **kwargs):
+    def indexing_func(self: TradesT, pd_indexing_func: tp.PandasIndexingFunc, **kwargs) -> TradesT:
         """Perform indexing on `Trades`."""
-        return self._indexing_func_meta(pd_indexing_func, **kwargs)[0]
+        return self.indexing_func_meta(pd_indexing_func, **kwargs)[0]
 
     @property
-    def close(self):
+    def close(self) -> tp.SeriesFrame:
         """Reference price such as close."""
         return self._close
 
     @property
-    def trade_type(self):
+    def trade_type(self) -> int:
         """Trade type."""
         return self._trade_type
 
     @classmethod
-    def from_orders(cls, orders, **kwargs):
+    def from_orders(cls: tp.Type[TradesT], orders: Orders, **kwargs) -> TradesT:
         """Build `Trades` from `vectorbt.portfolio.orders.Orders`."""
         trade_records_arr = nb.orders_to_trades_nb(
             orders.close.vbt.to_2d_array(),
@@ -187,7 +198,7 @@ class Trades(Records):
         return cls(orders.wrapper, trade_records_arr, orders.close, **kwargs)
 
     @property  # no need for cached
-    def records_readable(self):
+    def records_readable(self) -> tp.Frame:
         """Records in readable format."""
         records_df = self.records
         out = pd.DataFrame()
@@ -210,30 +221,30 @@ class Trades(Records):
         return out
 
     @cached_property
-    def duration(self):
+    def duration(self) -> MappedArray:
         """Duration of each trade (in raw format)."""
         return self.map(nb.trade_duration_map_nb)
 
     @cached_property
-    def pnl(self):
+    def pnl(self) -> MappedArray:
         """PnL of each trade."""
         return self.map_field('pnl')
 
     @cached_property
-    def returns(self):
+    def returns(self) -> MappedArray:
         """Return of each trade."""
         return self.map_field('return')
 
     # ############# PnL ############# #
 
     @cached_property
-    def winning(self):
+    def winning(self: TradesT) -> TradesT:
         """Winning trades."""
         filter_mask = self.values['pnl'] > 0.
         return self.filter_by_mask(filter_mask)
 
     @cached_method
-    def win_rate(self, group_by=None, wrap_kwargs=None):
+    def win_rate(self, group_by: tp.GroupByLike = None, wrap_kwargs: tp.KwargsLike = None) -> tp.MaybeSeries:
         """Rate of winning trades."""
         win_count = to_1d(self.winning.count(group_by=group_by), raw=True)
         total_count = to_1d(self.count(group_by=group_by), raw=True)
@@ -241,13 +252,13 @@ class Trades(Records):
         return self.wrapper.wrap_reduced(win_count / total_count, group_by=group_by, **wrap_kwargs)
 
     @cached_property
-    def losing(self):
+    def losing(self: TradesT) -> TradesT:
         """Losing trades."""
         filter_mask = self.values['pnl'] < 0.
         return self.filter_by_mask(filter_mask)
 
     @cached_method
-    def loss_rate(self, group_by=None, wrap_kwargs=None):
+    def loss_rate(self, group_by: tp.GroupByLike = None, wrap_kwargs: tp.KwargsLike = None) -> tp.MaybeSeries:
         """Rate of losing trades."""
         loss_count = to_1d(self.losing.count(group_by=group_by), raw=True)
         total_count = to_1d(self.count(group_by=group_by), raw=True)
@@ -255,7 +266,7 @@ class Trades(Records):
         return self.wrapper.wrap_reduced(loss_count / total_count, group_by=group_by, **wrap_kwargs)
 
     @cached_method
-    def profit_factor(self, group_by=None, wrap_kwargs=None):
+    def profit_factor(self, group_by: tp.GroupByLike = None, wrap_kwargs: tp.KwargsLike = None) -> tp.MaybeSeries:
         """Profit factor."""
         total_win = to_1d(self.winning.pnl.sum(group_by=group_by), raw=True)
         total_loss = to_1d(self.losing.pnl.sum(group_by=group_by), raw=True)
@@ -270,7 +281,7 @@ class Trades(Records):
         return self.wrapper.wrap_reduced(profit_factor, group_by=group_by, **wrap_kwargs)
 
     @cached_method
-    def expectancy(self, group_by=None, wrap_kwargs=None):
+    def expectancy(self, group_by: tp.GroupByLike = None, wrap_kwargs: tp.KwargsLike = None) -> tp.MaybeSeries:
         """Average profitability."""
         win_rate = to_1d(self.win_rate(group_by=group_by), raw=True)
         avg_win = to_1d(self.winning.pnl.mean(group_by=group_by), raw=True)
@@ -286,7 +297,7 @@ class Trades(Records):
         return self.wrapper.wrap_reduced(expectancy, group_by=group_by, **wrap_kwargs)
 
     @cached_method
-    def sqn(self, group_by=None, wrap_kwargs=None):
+    def sqn(self, group_by: tp.GroupByLike = None, wrap_kwargs: tp.KwargsLike = None) -> tp.MaybeSeries:
         """System Quality Number (SQN)."""
         count = to_1d(self.count(group_by=group_by), raw=True)
         pnl_mean = to_1d(self.pnl.mean(group_by=group_by), raw=True)
@@ -298,18 +309,18 @@ class Trades(Records):
     # ############# TradeDirection ############# #
 
     @cached_property
-    def direction(self):
+    def direction(self) -> MappedArray:
         """See `vectorbt.portfolio.enums.TradeDirection`."""
         return self.map_field('direction')
 
     @cached_property
-    def long(self):
+    def long(self: TradesT) -> TradesT:
         """Long trades."""
         filter_mask = self.values['direction'] == TradeDirection.Long
         return self.filter_by_mask(filter_mask)
 
     @cached_method
-    def long_rate(self, group_by=None, wrap_kwargs=None):
+    def long_rate(self, group_by: tp.GroupByLike = None, wrap_kwargs: tp.KwargsLike = None) -> tp.MaybeSeries:
         """Rate of long trades."""
         long_count = to_1d(self.long.count(group_by=group_by), raw=True)
         total_count = to_1d(self.count(group_by=group_by), raw=True)
@@ -317,13 +328,13 @@ class Trades(Records):
         return self.wrapper.wrap_reduced(long_count / total_count, group_by=group_by, **wrap_kwargs)
 
     @cached_property
-    def short(self):
+    def short(self: TradesT) -> TradesT:
         """Short trades."""
         filter_mask = self.values['direction'] == TradeDirection.Short
         return self.filter_by_mask(filter_mask)
 
     @cached_method
-    def short_rate(self, group_by=None, wrap_kwargs=None):
+    def short_rate(self, group_by: tp.GroupByLike = None, wrap_kwargs: tp.KwargsLike = None) -> tp.MaybeSeries:
         """Rate of short trades."""
         short_count = to_1d(self.short.count(group_by=group_by), raw=True)
         total_count = to_1d(self.count(group_by=group_by), raw=True)
@@ -333,18 +344,18 @@ class Trades(Records):
     # ############# TradeStatus ############# #
 
     @cached_property
-    def status(self):
+    def status(self) -> MappedArray:
         """See `vectorbt.portfolio.enums.TradeStatus`."""
         return self.map_field('status')
 
     @cached_property
-    def open(self):
+    def open(self: TradesT) -> TradesT:
         """Open trades."""
         filter_mask = self.values['status'] == TradeStatus.Open
         return self.filter_by_mask(filter_mask)
 
     @cached_method
-    def open_rate(self, group_by=None, wrap_kwargs=None):
+    def open_rate(self, group_by: tp.GroupByLike = None, wrap_kwargs: tp.KwargsLike = None) -> tp.MaybeSeries:
         """Rate of open trades."""
         open_count = to_1d(self.open.count(group_by=group_by), raw=True)
         total_count = to_1d(self.count(group_by=group_by), raw=True)
@@ -352,13 +363,13 @@ class Trades(Records):
         return self.wrapper.wrap_reduced(open_count / total_count, group_by=group_by, **wrap_kwargs)
 
     @cached_property
-    def closed(self):
+    def closed(self: TradesT) -> TradesT:
         """Closed trades."""
         filter_mask = self.values['status'] == TradeStatus.Closed
         return self.filter_by_mask(filter_mask)
 
     @cached_method
-    def closed_rate(self, group_by=None, wrap_kwargs=None):
+    def closed_rate(self, group_by: tp.GroupByLike = None, wrap_kwargs: tp.KwargsLike = None) -> tp.MaybeSeries:
         """Rate of closed trades."""
         closed_count = to_1d(self.closed.count(group_by=group_by), raw=True)
         total_count = to_1d(self.count(group_by=group_by), raw=True)
@@ -367,22 +378,25 @@ class Trades(Records):
 
     # ############# Plotting ############# #
 
-    def plot_pnl(self,
-                 column=None,
-                 marker_size_range=(7, 14),
-                 opacity_range=(0.75, 0.9),
-                 closed_profit_trace_kwargs=None,
-                 closed_loss_trace_kwargs=None,
-                 open_trace_kwargs=None,
-                 hline_shape_kwargs=None,
-                 add_trace_kwargs=None,
-                 xref='x', yref='y',
-                 fig=None,
-                 **layout_kwargs):  # pragma: no cover
+    def plot_pnl_returns(self,
+                         column: tp.Optional[tp.Label] = None,
+                         as_pct: bool = True,
+                         marker_size_range: tp.Tuple[float, float] = (7, 14),
+                         opacity_range: tp.Tuple[float, float] = (0.75, 0.9),
+                         closed_profit_trace_kwargs: tp.KwargsLike = None,
+                         closed_loss_trace_kwargs: tp.KwargsLike = None,
+                         open_trace_kwargs: tp.KwargsLike = None,
+                         hline_shape_kwargs: tp.KwargsLike = None,
+                         add_trace_kwargs: tp.KwargsLike = None,
+                         xref: str = 'x',
+                         yref: str = 'y',
+                         fig: tp.Optional[tp.BaseFigure] = None,
+                         **layout_kwargs) -> tp.BaseFigure:  # pragma: no cover
         """Plot trade PnL.
 
         Args:
             column (str): Name of the column to plot.
+            as_pct (bool): Whether to set y-axis to `Trades.returns`, otherwise to `Trades.pnl`.
             marker_size_range (tuple): Range of marker size.
             opacity_range (tuple): Range of marker opacity.
             closed_profit_trace_kwargs (dict): Keyword arguments passed to `plotly.graph_objects.Scatter` for "Closed - Profit" markers.
@@ -392,16 +406,8 @@ class Trades(Records):
             add_trace_kwargs (dict): Keyword arguments passed to `add_trace`.
             xref (str): X coordinate axis.
             yref (str): Y coordinate axis.
-            fig (plotly.graph_objects.Figure): Figure to add traces to.
+            fig (Figure or FigureWidget): Figure to add traces to.
             **layout_kwargs: Keyword arguments for layout.
-
-        ## Example
-
-        ```python-repl
-        >>> trades.plot_pnl()
-        ```
-
-        ![](/vectorbt/docs/img/trades_plot_pnl.png)
         """
         from vectorbt.settings import contrast_color_schema
 
@@ -418,12 +424,17 @@ class Trades(Records):
         if add_trace_kwargs is None:
             add_trace_kwargs = {}
         marker_size_range = tuple(marker_size_range)
+        xaxis = 'xaxis' + xref[1:]
+        yaxis = 'yaxis' + yref[1:]
 
         if fig is None:
-            fig = FigureWidget()
+            fig = make_figure()
+        if as_pct:
+            _layout_kwargs = dict()
+            _layout_kwargs[yaxis] = dict(tickformat='.2%')
+            fig.update_layout(**_layout_kwargs)
         fig.update_layout(**layout_kwargs)
         x_domain = [0, 1]
-        xaxis = 'xaxis' + xref[1:]
         if xaxis in fig.layout:
             if 'domain' in fig.layout[xaxis]:
                 if fig.layout[xaxis]['domain'] is not None:
@@ -433,6 +444,8 @@ class Trades(Records):
             # Extract information
             _id = self.values['id']
             _id_str = 'Trade Id' if self.trade_type == TradeType.Trade else 'Position Id'
+            _pnl_str = '%{customdata[1]:.6f}' if as_pct else '%{y}'
+            _return_str = '%{y}' if as_pct else '%{customdata[1]:.2%}'
             exit_idx = self.values['exit_idx']
             pnl = self.values['pnl']
             returns = self.values['return']
@@ -450,83 +463,58 @@ class Trades(Records):
             closed_loss_mask = (~open_mask) & loss_mask
             open_mask &= ~neutral_mask
 
-            if np.any(closed_profit_mask):
-                # Plot Profit markers
-                profit_scatter = go.Scatter(
-                    x=self_col.wrapper.index[exit_idx[closed_profit_mask]],
-                    y=pnl[closed_profit_mask],
-                    mode='markers',
-                    marker=dict(
-                        symbol='circle',
-                        color=contrast_color_schema['green'],
-                        size=marker_size[closed_profit_mask],
-                        opacity=opacity[closed_profit_mask],
-                        line=dict(
-                            width=1,
-                            color=adjust_lightness(contrast_color_schema['green'])
+            def _plot_scatter(mask: tp.Array1d, name: tp.TraceName, color: tp.Any, kwargs: tp.Kwargs) -> None:
+                if np.any(mask):
+                    scatter = go.Scatter(
+                        x=self_col.wrapper.index[exit_idx[mask]],
+                        y=returns[mask] if as_pct else pnl[mask],
+                        mode='markers',
+                        marker=dict(
+                            symbol='circle',
+                            color=color,
+                            size=marker_size[mask],
+                            opacity=opacity[mask],
+                            line=dict(
+                                width=1,
+                                color=adjust_lightness(color)
+                            ),
                         ),
-                    ),
-                    name='Closed - Profit',
-                    customdata=np.stack((_id[closed_profit_mask], returns[closed_profit_mask]), axis=1),
-                    hovertemplate=_id_str + ": %{customdata[0]}"
-                                            "<br>Date: %{x}"
-                                            "<br>PnL: %{y}"
-                                            "<br>Return: %{customdata[1]:.2%}"
-                )
-                profit_scatter.update(**closed_profit_trace_kwargs)
-                fig.add_trace(profit_scatter, **add_trace_kwargs)
+                        name=name,
+                        customdata=np.stack((
+                            _id[mask],
+                            pnl[mask] if as_pct else returns[mask]
+                        ), axis=1),
+                        hovertemplate=_id_str + ": %{customdata[0]}"
+                                                "<br>Date: %{x}"
+                                                f"<br>PnL: {_pnl_str}"
+                                                f"<br>Return: {_return_str}"
+                    )
+                    scatter.update(**kwargs)
+                    fig.add_trace(scatter, **add_trace_kwargs)
 
-            if np.any(closed_loss_mask):
-                # Plot Loss markers
-                loss_scatter = go.Scatter(
-                    x=self_col.wrapper.index[exit_idx[closed_loss_mask]],
-                    y=pnl[closed_loss_mask],
-                    mode='markers',
-                    marker=dict(
-                        symbol='circle',
-                        color=contrast_color_schema['red'],
-                        size=marker_size[closed_loss_mask],
-                        opacity=opacity[closed_loss_mask],
-                        line=dict(
-                            width=1,
-                            color=adjust_lightness(contrast_color_schema['red'])
-                        )
-                    ),
-                    name='Closed - Loss',
-                    customdata=np.stack((_id[closed_loss_mask], returns[closed_loss_mask]), axis=1),
-                    hovertemplate=_id_str + ": %{customdata[0]}"
-                                            "<br>Date: %{x}"
-                                            "<br>PnL: %{y}"
-                                            "<br>Return: %{customdata[1]:.2%}"
-                )
-                loss_scatter.update(**closed_loss_trace_kwargs)
-                fig.add_trace(loss_scatter, **add_trace_kwargs)
+            # Plot Closed - Profit scatter
+            _plot_scatter(
+                closed_profit_mask,
+                'Closed - Profit',
+                contrast_color_schema['green'],
+                closed_profit_trace_kwargs
+            )
 
-            if np.any(open_mask):
-                # Plot Active markers
-                active_scatter = go.Scatter(
-                    x=self_col.wrapper.index[exit_idx[open_mask]],
-                    y=pnl[open_mask],
-                    mode='markers',
-                    marker=dict(
-                        symbol='circle',
-                        color=contrast_color_schema['orange'],
-                        size=marker_size[open_mask],
-                        opacity=opacity[open_mask],
-                        line=dict(
-                            width=1,
-                            color=adjust_lightness(contrast_color_schema['orange'])
-                        )
-                    ),
-                    name='Open',
-                    customdata=np.stack((_id[open_mask], returns[open_mask]), axis=1),
-                    hovertemplate=_id_str + ": %{customdata[0]}"
-                                            "<br>Date: %{x}"
-                                            "<br>PnL: %{y}"
-                                            "<br>Return: %{customdata[1]:.2%}"
-                )
-                active_scatter.update(**open_trace_kwargs)
-                fig.add_trace(active_scatter, **add_trace_kwargs)
+            # Plot Closed - Profit scatter
+            _plot_scatter(
+                closed_loss_mask,
+                'Closed - Loss',
+                contrast_color_schema['red'],
+                closed_loss_trace_kwargs
+            )
+
+            # Plot Open scatter
+            _plot_scatter(
+                open_mask,
+                'Open',
+                contrast_color_schema['orange'],
+                open_trace_kwargs
+            )
 
         # Plot zeroline
         fig.add_shape(**merge_dicts(dict(
@@ -544,22 +532,39 @@ class Trades(Records):
         ), hline_shape_kwargs))
         return fig
 
+    def plot_pnl(self, **kwargs):
+        """`Trades.plot_pnl_returns` with `as_pct` set to False.
+
+        ## Example
+
+        ```python-repl
+        >>> trades.plot_pnl()
+        ```
+
+        ![](/vectorbt/docs/img/trades_plot_pnl.svg)"""
+        return self.plot_pnl_returns(as_pct=False, **kwargs)
+
+    def plot_returns(self, **kwargs):
+        """`Trades.plot_pnl_returns` with `as_pct` set to True."""
+        return self.plot_pnl_returns(as_pct=True, **kwargs)
+
     def plot(self,
-             column=None,
-             plot_close=True,
-             plot_zones=True,
-             close_trace_kwargs=None,
-             entry_trace_kwargs=None,
-             exit_trace_kwargs=None,
-             exit_profit_trace_kwargs=None,
-             exit_loss_trace_kwargs=None,
-             active_trace_kwargs=None,
-             profit_shape_kwargs=None,
-             loss_shape_kwargs=None,
-             add_trace_kwargs=None,
-             xref='x', yref='y',
-             fig=None,
-             **layout_kwargs):  # pragma: no cover
+             column: tp.Optional[tp.Label] = None,
+             plot_close: bool = True,
+             plot_zones: bool = True,
+             close_trace_kwargs: tp.KwargsLike = None,
+             entry_trace_kwargs: tp.KwargsLike = None,
+             exit_trace_kwargs: tp.KwargsLike = None,
+             exit_profit_trace_kwargs: tp.KwargsLike = None,
+             exit_loss_trace_kwargs: tp.KwargsLike = None,
+             active_trace_kwargs: tp.KwargsLike = None,
+             profit_shape_kwargs: tp.KwargsLike = None,
+             loss_shape_kwargs: tp.KwargsLike = None,
+             add_trace_kwargs: tp.KwargsLike = None,
+             xref: str = 'x',
+             yref: str = 'y',
+             fig: tp.Optional[tp.BaseFigure] = None,
+             **layout_kwargs) -> tp.BaseFigure:  # pragma: no cover
         """Plot orders.
 
         Args:
@@ -579,7 +584,7 @@ class Trades(Records):
             add_trace_kwargs (dict): Keyword arguments passed to `add_trace`.
             xref (str): X coordinate axis.
             yref (str): Y coordinate axis.
-            fig (plotly.graph_objects.Figure): Figure to add traces to.
+            fig (Figure or FigureWidget): Figure to add traces to.
             **layout_kwargs: Keyword arguments for layout.
 
         ## Example
@@ -588,7 +593,7 @@ class Trades(Records):
         >>> trades.plot()
         ```
 
-        ![](/vectorbt/docs/img/trades_plot.png)"""
+        ![](/vectorbt/docs/img/trades_plot.svg)"""
         from vectorbt.settings import color_schema, contrast_color_schema
 
         self_col = self.select_series(column=column, group_by=False)
@@ -617,7 +622,7 @@ class Trades(Records):
             add_trace_kwargs = {}
 
         if fig is None:
-            fig = FigureWidget()
+            fig = make_figure()
         fig.update_layout(**layout_kwargs)
 
         # Plot close
@@ -642,8 +647,8 @@ class Trades(Records):
             direction = np.vectorize(lambda x: str(direction_value_map[x]))(direction)
             status = self_col.values['status']
 
-            def get_duration_str(from_idx, to_idx):
-                if isinstance(self_col.wrapper.index, DatetimeTypes):
+            def _get_duration_str(from_idx: int, to_idx: int) -> tp.Array1d:
+                if isinstance(self_col.wrapper.index, DatetimeIndexes):
                     duration = self_col.wrapper.index[to_idx] - self_col.wrapper.index[from_idx]
                 elif self_col.wrapper.freq is not None:
                     duration = self_col.wrapper.to_time_units(to_idx - from_idx)
@@ -651,7 +656,7 @@ class Trades(Records):
                     duration = to_idx - from_idx
                 return np.vectorize(str)(duration)
 
-            duration = get_duration_str(entry_idx, exit_idx)
+            duration = _get_duration_str(entry_idx, exit_idx)
 
             if len(entry_idx) > 0:
                 # Plot Entry markers
@@ -691,7 +696,7 @@ class Trades(Records):
                 fig.add_trace(entry_scatter, **add_trace_kwargs)
 
             # Plot end markers
-            def _plot_end_markers(mask, name, color, kwargs):
+            def _plot_end_markers(mask: tp.Array1d, name: tp.TraceName, color: tp.Any, kwargs: tp.Kwargs) -> None:
                 if np.any(mask):
                     customdata = np.stack((
                         _id[mask],
@@ -809,6 +814,9 @@ class Trades(Records):
 # ############# Positions ############# #
 
 
+PositionsT = tp.TypeVar("PositionsT", bound="Positions")
+
+
 class Positions(Trades):
     """Extends `Trades` for working with position records.
 
@@ -866,23 +874,23 @@ class Positions(Trades):
     ```
     """
 
-    def __init__(self, *args, trade_type=TradeType.Position, **kwargs):
+    def __init__(self, *args, trade_type: int = TradeType.Position, **kwargs) -> None:
         if trade_type != TradeType.Position:
             raise ValueError("Trade type must be TradeType.Position")
         Trades.__init__(self, *args, trade_type=trade_type, **kwargs)
 
     @classmethod
-    def from_orders(cls, orders, **kwargs):
+    def from_orders(cls: tp.Type[PositionsT], orders: Orders, **kwargs) -> PositionsT:
         raise NotImplementedError
 
     @classmethod
-    def from_trades(cls, trades, **kwargs):
+    def from_trades(cls: tp.Type[PositionsT], trades: Trades, **kwargs) -> PositionsT:
         """Build `Positions` from `Trades`."""
         position_records_arr = nb.trades_to_positions_nb(trades.values, trades.col_mapper.col_map)
         return cls(trades.wrapper, position_records_arr, trades.close, **kwargs)
 
     @cached_method
-    def coverage(self, group_by=None, wrap_kwargs=None):
+    def coverage(self, group_by: tp.GroupByLike = None, wrap_kwargs: tp.KwargsLike = None) -> tp.SeriesFrame:
         """Coverage, that is, total duration divided by the whole period."""
         total_duration = to_1d(self.duration.sum(group_by=group_by), raw=True)
         total_steps = self.wrapper.grouper.get_group_lens(group_by=group_by) * self.wrapper.shape[0]
