@@ -1,18 +1,20 @@
 """Class for mapping column arrays."""
 
+from vectorbt import typing as tp
 from vectorbt.utils.decorators import cached_property, cached_method
 from vectorbt.base.reshape_fns import to_1d
+from vectorbt.base.array_wrapper import ArrayWrapper
 from vectorbt.records import nb
 
 
 class ColumnMapper:
     """Used by `vectorbt.records.base.Records` and `vectorbt.records.mapped_array.MappedArray`
     classes to make use of column and group metadata."""
-    def __init__(self, wrapper, col_arr):
+    def __init__(self, wrapper: ArrayWrapper, col_arr: tp.Array1d) -> None:
         self._wrapper = wrapper
         self._col_arr = col_arr
 
-    def _col_idxs_meta(self, col_idxs):
+    def _col_idxs_meta(self, col_idxs: tp.Array1d) -> tp.Tuple[tp.Array1d, tp.Array1d]:
         """Get metadata of column indices.
 
         Returns element indices and new column array.
@@ -24,17 +26,17 @@ class ColumnMapper:
         return new_indices, new_col_arr
 
     @property
-    def wrapper(self):
+    def wrapper(self) -> ArrayWrapper:
         """Array wrapper."""
         return self._wrapper
 
     @property
-    def col_arr(self):
+    def col_arr(self) -> tp.Array1d:
         """Column array."""
         return self._col_arr
 
     @cached_method
-    def get_col_arr(self, group_by=None):
+    def get_col_arr(self, group_by: tp.GroupByLike = None) -> tp.Array1d:
         """Get group-aware column array."""
         group_arr = self.wrapper.grouper.get_groups(group_by=group_by)
         if group_arr is not None:
@@ -44,7 +46,7 @@ class ColumnMapper:
         return col_arr
 
     @cached_property
-    def col_range(self):
+    def col_range(self) -> tp.ColRange:
         """Column index.
 
         Faster than `ColumnMapper.col_map` but only compatible with sorted columns.
@@ -52,7 +54,7 @@ class ColumnMapper:
         return nb.col_range_nb(self.col_arr, len(self.wrapper.columns))
 
     @cached_method
-    def get_col_range(self, group_by=None):
+    def get_col_range(self, group_by: tp.GroupByLike = None) -> tp.ColRange:
         """Get group-aware column range."""
         if not self.wrapper.grouper.is_grouped(group_by=group_by):
             return self.col_range
@@ -61,7 +63,7 @@ class ColumnMapper:
         return nb.col_range_nb(col_arr, len(columns))
 
     @cached_property
-    def col_map(self):
+    def col_map(self) -> tp.ColMap:
         """Column map.
 
         More flexible than `ColumnMapper.col_range`.
@@ -69,7 +71,7 @@ class ColumnMapper:
         return nb.col_map_nb(self.col_arr, len(self.wrapper.columns))
 
     @cached_method
-    def get_col_map(self, group_by=None):
+    def get_col_map(self, group_by: tp.GroupByLike = None) -> tp.ColMap:
         """Get group-aware column map."""
         if not self.wrapper.grouper.is_grouped(group_by=group_by):
             return self.col_map
@@ -78,7 +80,7 @@ class ColumnMapper:
         return nb.col_map_nb(col_arr, len(columns))
 
     @cached_method
-    def is_sorted(self):
+    def is_sorted(self) -> bool:
         """Check whether column array is sorted."""
         return nb.is_col_sorted_nb(self.col_arr)
 
