@@ -3,14 +3,17 @@
 import numpy as np
 from scipy.stats import norm
 
+from vectorbt import typing as tp
 
-def approx_exp_max_sharpe(mean_sharpe, var_sharpe, nb_trials):
+
+def approx_exp_max_sharpe(mean_sharpe: float, var_sharpe: float, nb_trials: int) -> float:
     """Expected Maximum Sharpe Ratio."""
     return mean_sharpe + np.sqrt(var_sharpe) * \
         ((1 - np.euler_gamma) * norm.ppf(1 - 1 / nb_trials) + np.euler_gamma * norm.ppf(1 - 1 / (nb_trials * np.e)))
 
 
-def deflated_sharpe_ratio(*, est_sharpe, var_sharpe, nb_trials, backtest_horizon, skew, kurtosis):
+def deflated_sharpe_ratio(*, est_sharpe: tp.Array1d, var_sharpe: float, nb_trials: int,
+                          backtest_horizon: int, skew: tp.Array1d, kurtosis: tp.Array1d) -> tp.Array1d:
     """Deflated Sharpe Ratio (DSR).
 
     See [Deflated Sharpe Ratio](https://gmarti.gitlab.io/qfin/2018/05/30/deflated-sharpe-ratio.html)."""
