@@ -31,6 +31,7 @@ So, for example, the method `pd.Series.vbt.to_2d_array` is also available as
 import pandas as pd
 from pandas.core.accessor import _register_accessor, DirNamesMixin
 
+from vectorbt import typing as tp
 from vectorbt.generic.accessors import GenericSRAccessor, GenericDFAccessor
 
 
@@ -39,7 +40,7 @@ from vectorbt.generic.accessors import GenericSRAccessor, GenericDFAccessor
 class Vbt_SRAccessor(DirNamesMixin, GenericSRAccessor):
     """The main vectorbt accessor for `pd.Series`."""
 
-    def __init__(self, obj, **kwargs):
+    def __init__(self, obj: tp.Series, **kwargs) -> None:
         self._obj = obj
 
         DirNamesMixin.__init__(self)
@@ -50,18 +51,18 @@ class Vbt_SRAccessor(DirNamesMixin, GenericSRAccessor):
 class Vbt_DFAccessor(DirNamesMixin, GenericDFAccessor):
     """The main vectorbt accessor for `pd.DataFrame`."""
 
-    def __init__(self, obj, **kwargs):
+    def __init__(self, obj: tp.Frame, **kwargs) -> None:
         self._obj = obj
 
         DirNamesMixin.__init__(self)
         GenericDFAccessor.__init__(self, obj, **kwargs)
 
 
-def register_dataframe_accessor(name):
+def register_dataframe_accessor(name: str) -> tp.Callable:
     """Decorator to register a custom `pd.DataFrame` accessor on top of the `vbt` accessor."""
     return _register_accessor(name, Vbt_DFAccessor)
 
 
-def register_series_accessor(name):
+def register_series_accessor(name: str) -> tp.Callable:
     """Decorator to register a custom `pd.Series` accessor on top of the `vbt` accessor."""
     return _register_accessor(name, Vbt_SRAccessor)
