@@ -2,11 +2,10 @@
 
 from vectorbt.utils.config import atomic_dict, merge_dicts, Config, Configured, AtomicConfig
 from vectorbt.utils.decorators import cached_property, cached_method
-from vectorbt.utils.widgets import Figure, FigureWidget, make_subplots
+from vectorbt.utils.figure import Figure, FigureWidget, make_figure, make_subplots
 from vectorbt.utils.random import set_seed
 from vectorbt.utils.image import save_animation
 from vectorbt.utils.schedule import AsyncJob, AsyncScheduler, CancelledError, ScheduleManager
-from vectorbt.utils.messaging import TelegramBot
 
 __all__ = [
     'atomic_dict',
@@ -18,14 +17,14 @@ __all__ = [
     'cached_method',
     'Figure',
     'FigureWidget',
+    'make_figure',
     'make_subplots',
     'set_seed',
     'save_animation',
     'AsyncJob',
     'AsyncScheduler',
     'CancelledError',
-    'ScheduleManager',
-    'TelegramBot'
+    'ScheduleManager'
 ]
 
 __pdoc__ = {k: False for k in __all__}
